/*
*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 1994 - Barry W. Brown, James Lovato, Kathy Russell (DCDFLIB)
*
*/

/* cdfbet.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
*/

#include <math.h>
#include <stdio.h>

#include "cdflib.h"
#include "cdflib_private.h" 
#include "brent.h"


/* ********************************************************************** */

// Beta Distribution

/*                              Method */


/*     Cumulative distribution function  (P)  is calculated directly by */
/*     code associated with the following reference. */

/*     DiDinato, A. R. and Morris,  A.   H.  Algorithm 708: Significant */
/*     Digit Computation of the Incomplete  Beta  Function Ratios.  ACM */
/*     Trans. Math.  Softw. 18 (1993), 360-373. */

/*     Computation of other parameters involve a seach for a value that */
/*     produces  the desired  value  of P.   The search relies  on  the */
/*     monotinicity of P with the other parameter. */

/*                              Note */

/*     The beta density is proportional to */
/*               t^(A-1) * (1-t)^(B-1) */

/* ********************************************************************** */

int cdflib_betCheckParams(char* fname, double a, double b)
{
	int status;
	/*     A > 0 */
	status=cdflib_checkgreaterthan(fname, a, "a", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	/*     B > 0 */
	status=cdflib_checkgreaterthan(fname, b, "b", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}

int cdflib_betCheckXY(char* fname, double x, double y)
{
	/*     X, Y, X + Y */
	int status;
	status=cdflib_checkpq(fname, x, "x", y, "y");
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}
int cdflib_betCheckX(char* fname, double x)
{
	/*     X in [0,1] */
	int status;
	status=cdflib_checkrangedouble(fname, x, "x", 0., 1.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}

int cdflib_betainv(double p, double a, double b, int lowertail, double *x)
{
	int status;
	static int qporq;

	static double fx;
	static double cum, xhi, xlo, ccum;

	double atol = cdflib_doubleTiny();
	double q;
	double y;
	int iteration;

	// P in [0,1]
	status=cdflib_checkp("cdflib_betainv", p, "p");
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Check A, B
	status=cdflib_betCheckParams("cdflib_betainv",a, b);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Compute the couple (p,q) from (p,lowertail)
	if (lowertail==CDFLIB_LOWERTAIL)
	{
		q=1-p;
	}
	else
	{
		q=p;
		p=1-q;
	}
	/*     Select the minimum of P or Q */
	qporq = p <= q;
	/*     Calculating X and Y */
	if ( p == 0. ) 
	{
		// Baudin - 2012 - Fix bug #9976
		*x = 0.;
		status = CDFLIB_OK;
		return status;
	}
	if ( q == 0. ) 
	{
		// Baudin - 2012 - Fix bug #896
		*x = 1.;
		status = CDFLIB_OK;
		return status;
	}
	if (cdflib_isnan(p) || cdflib_isnan(q) || cdflib_isnan(a) || cdflib_isnan(b))
	{
		*x = p+q+a+b;
		status = CDFLIB_OK;
		return status;
	}
	if (qporq)
	{
		y = 1. - *x;
	}
	status = 0;
	iteration=0;
	while (1)
	{
		if (qporq)
		{
			zero_rc ( 0., 1., atol, x, &status, fx );
			y = 1. - *x;
		}
		else
		{
			zero_rc ( 0., 1., atol, &y, &status, fx );
			*x = 1. - y;
		}
		if (status<0) 
		{
			break;
		}
		cdflib_cumbet(*x, y, a, b, &cum, &ccum);
		fx = cdflib_computefx(p, q, cum, ccum);
		if (status==0) 
		{
			break;
		}
		iteration=iteration+1;
	}
	if (status == 0) 
	{
		status = CDFLIB_OK;
	}
	else
	{
		cdflib_unabletoinvert("cdflib_betainv", *x, "x");
		status = CDFLIB_ERROR;
	}
	cdflib_printiter("cdflib_betainv", iteration);
	return status;
}
int cdflib_betacdf(double x, double a, double b, int lowertail, double *p)
{
	int status;
	double q;
	double y;

	status=cdflib_betCheckX("cdflib_betcdf",x);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	status=cdflib_betCheckParams("cdflib_betcdf",a, b);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	/*     Calculating P and Q */
	if (cdflib_isnan(x) || cdflib_isnan(a) || cdflib_isnan(b))
	{
		*p = x+a+b;
		status = CDFLIB_OK;
		return status;
	}
	// Compute y
	y=1-x;
	cdflib_cumbet(x, y, a, b, p, &q);
	if (lowertail==CDFLIB_UPPERTAIL)
	{
		*p=q;
	}
	status = CDFLIB_OK;
	return status;
}
