/*
*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 1994 - Barry W. Brown, James Lovato, Kathy Russell (DCDFLIB)
*
*/

/* cdfbin.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
*/

#include <math.h>
#include <stdio.h>

#include "cdflib.h"
#include "cdflib_private.h" 
#include "brent.h"


/* ********************************************************************** */

// Binomial Distribution

/*                              Method */

/*     Formula  26.5.24    of   Abramowitz  and    Stegun,  Handbook   of */
/*     Mathematical   Functions (1966) is   used  to reduce the  binomial */
/*     distribution  to  the  cumulative incomplete    beta distribution. */

/*     Computation of other parameters involve a seach for a value that */
/*     produces  the desired  value  of P.   The search relies  on  the */
/*     monotinicity of P with the other parameter. */

/* ********************************************************************** */

int cdflib_binCheckParams(char * fname, double n, double pr)
{
	int status;
	// N has integer value
	status=cdflib_checkIntValue(fname, n, "N");
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	/*     N > 0 */
	status=cdflib_checkgreaterthan(fname, n, "N", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	//     PR in [0,1]
	status=cdflib_checkp(fname, pr, "pr");
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}
int cdflib_binCheckX(char * fname, double x, double n)
{
	int status;
	// X has integer value
	status=cdflib_checkIntValue(fname, x, "x");
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	/*     X >=0 and X<=N */
	status = cdflib_checkgreqthan(fname, x, "X", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	status = cdflib_checkloweqthan(fname, x, "X", n);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}

int cdflib_binocdf(double x, double n, double pr, int lowertail, double *p)
{
	int status;
	double ompr;
	double q;

	status = cdflib_binCheckX("cdflib_binocdf", x, n);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Check n, pr, ompr
	status=cdflib_binCheckParams("cdflib_binocdf",n,pr);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	ompr=1-pr;
	/*     Calculating P and Q */
	if (cdflib_isnan(x) || cdflib_isnan(n) || cdflib_isnan(pr))
	{
		*p = x+n+pr;
		status = CDFLIB_OK;
		return status;
	}
	cdflib_cumbin(x, n, pr, ompr, p, &q);
	if (lowertail==CDFLIB_UPPERTAIL)
	{
		*p=q;
	}
	status = CDFLIB_OK;
	return CDFLIB_OK;
}
int cdflib_binoinv(double p, double n, double pr, int lowertail, double *x)
{
	int status;
	double ompr;
	double q;

	/* Local variables */
	static double fx;
	static double cum, ccum;

	double rtol = cdflib_doubleEps();
	int iteration;

	// P in [0,1]
	status=cdflib_checkp("cdflib_binoinv", p, "p");
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Check n, pr
	status=cdflib_binCheckParams("cdflib_binoinv",n,pr);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Compute the couple (p,q) from (p,lowertail)
	if (lowertail==CDFLIB_LOWERTAIL)
	{
		q=1-p;
	}
	else
	{
		q=p;
		p=1-q;
	}
	ompr=1-pr;
	/*     Calculating X */
	if (cdflib_isnan(p) || cdflib_isnan(q) || cdflib_isnan(n) || cdflib_isnan(pr))
	{
		*x = p+q+n+pr;
		status = CDFLIB_OK;
		return status;
	}
	if (p==0)
	{
		*x = 0;
		status = CDFLIB_OK;
		return status;
	}
	if (q==0)
	{
		*x = n;
		status = CDFLIB_OK;
		return status;
	}
	//
	// For small values of p
	*x = 0;
	cdflib_cumbin(*x, n, pr, ompr, &cum, &ccum);
	fx = cdflib_computefx(p, q, cum, ccum);
	if (p<=cum)
	{
		// The value of p is too small to be 
		// attained by the CDF.
		// The solution is X=0
		status = CDFLIB_OK;
		return status;
	}
	//
	*x = 0.;
	status = 0;
	iteration=0;
	while (1)
	{
	    zero_rc ( 0., n, rtol, x, &status, fx );
		if (status<0) 
		{
			break;
		}
		cdflib_cumbin(*x, n, pr, ompr, &cum, &ccum);
		fx = cdflib_computefx(p, q, cum, ccum);
		if (status==0) 
		{
			break;
		}
		iteration=iteration+1;
	}
	if (status == 0) 
	{
		status = CDFLIB_OK;
		*x=ceil(*x);
	}
	else
	{
		cdflib_unabletoinvert("cdflib_binoinv", *x, "x");
		status = CDFLIB_ERROR;
	}
	cdflib_printiter("cdflib_binoinv", iteration);
	return status;
}
