/*
*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 1994 - Barry W. Brown, James Lovato, Kathy Russell (DCDFLIB)
*
*/

/* cdffnc.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
*/


#include "cdflib.h"
#include "cdflib_private.h" 
#include "brent.h"

/* ********************************************************************** */
/*               Cumulative Distribution Function */
/*               Non-central F distribution */
/*                              Function */
/*     Calculates any one parameter of the Non-central F */
/*     distribution given values for the others. */
/*                              Arguments */
/*     WHICH --> Integer indicating which of the next five argument */
/*               values is to be calculated from the others. */
/*               Legal range: 1,2 */
/*               iwhich = 1 : Calculate P and Q from F,DFN,DFD and PNONC */
/*               iwhich = 2 : Calculate F from P,Q,DFN,DFD and PNONC */
/*       P <--> The integral from 0 to F of the non-central f-density. */
/*              Input range: [0,1-1E-16). */
/*       Q <--> 1-P. */
/*            Input range: (0, 1]. */
/*            P + Q = 1.0. */
/*       F <--> Upper limit of integration of the non-central f-density. */
/*              Input range: [0, +infinity). */
/*              Search range: [0,huge] */
/*     DFN < --> Degrees of freedom of the numerator sum of squares. */
/*               Input range: (0, +infinity). */
/*               Search range: [ atol, huge] */
/*     DFD < --> Degrees of freedom of the denominator sum of squares. */
/*               Must be in range: (0, +infinity). */
/*               Input range: (0, +infinity). */
/*               Search range: [ atol, huge] */
/*     PNONC <-> The non-centrality parameter */
/*               Input range: [0,infinity) */
/*               Search range: [0,1E4] */
/*                              Method */
/*     Formula  26.6.20   of   Abramowitz   and   Stegun,  Handbook  of */
/*     Mathematical  Functions (1966) is used to compute the cumulative */
/*     distribution function. */
/*     Computation of other parameters involve a seach for a value that */
/*     produces  the desired  value  of P.   The search relies  on  the */
/*     monotinicity of P with the other parameter. */
/*                            WARNING */
/*     The computation time  required for this  routine is proportional */
/*     to the noncentrality  parameter  (PNONC).  Very huge  values of */
/*     this parameter can consume immense  computer resources.  This is */
/*     why the search range is bounded by 10,000. */
/*                              WARNING */
/*     The  value  of the  cumulative  noncentral F distribution is not */
/*     necessarily monotone in either degrees  of freedom.  There  thus */
/*     may be two values that provide a given  CDF value.  This routine */
/*     assumes monotonicity  and will find  an arbitrary one of the two */
/*     values. */
/* ********************************************************************** */

int cdflib_cdffnc(int which, double *p, double *q, 
	double *x, double dfn, double dfd, double phonc)
{
	int status;

	static double fx;
	static double cum, ccum;

	double huge = cdflib_doubleHuge();
	double atol = cdflib_doubleTiny();
	int iteration;
	double b;

	/*     Check arguments */

	status=cdflib_checkrangeint("cdflib_cdffnc", which, "which", 1, 2);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	/* P, Q, P + Q */
	if (which==2)
	{
		status=cdflib_checkpq("cdflib_cdffnc", *p, "p", *q, "q");
		if (status!=CDFLIB_OK)
		{
			return status;
		}
	}

	// Check X
	if (which==1)
	{
		status=cdflib_fncCheckX("cdflib_cdffnc", *x);
		if (status!=CDFLIB_OK)
		{
			return status;
		}
	}

	// Check DFN, DFD, PHONC
	status=cdflib_fncCheckParams("cdflib_cdffnc", dfn, dfd, phonc);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	/*     Calculate ANSWERS */
	if (which==1) 
	{
		/*     Calculating P and Q */
		if (cdflib_isnan(*x) || cdflib_isnan(dfn) || cdflib_isnan(dfd) || cdflib_isnan(phonc))
		{
			*p = *x+dfn+dfd+phonc;
			*q = *x+dfn+dfd+phonc;
			status = CDFLIB_OK;
			return status;
		}
		cdflib_cumfnc(*x, dfn, dfd, phonc, p, q);
		status = CDFLIB_OK;
	} 
	else if (which==2) 
	{
		/*     Calculating F */
		if (cdflib_isnan(*p) || cdflib_isnan(*q) || cdflib_isnan(dfn) || cdflib_isnan(dfd) || cdflib_isnan(phonc))
		{
			*x = *p+*q+dfn+dfd+phonc;
			status = CDFLIB_OK;
			return status;
		}
		//
		// Step A: Find a rough estimate of an interval.
		// Compute a "small" b, so that f(b) changes sign.
		b=cdflib_doubleTiny();
		iteration=0;
		while (1)
		{
			cdflib_cumfnc(b, dfn, dfd, phonc, &cum, &ccum);
			fx = cdflib_computefx(*p, *q, cum, ccum);
			if (*p <= *q && fx>0)
			{
				break;
			}
			if (*p > *q && fx<0)
			{
				break;
			}
			b=b*1.e10;
			iteration=iteration+1;
		}
		// Step B: Refine this estimate.
		status = 0;
		while (1)
		{
			zero_rc ( 0., b, atol, x, &status, fx );
			if (status<0) 
			{
				break;
			}
			cdflib_cumfnc(*x, dfn, dfd, phonc, &cum, &ccum);
			fx = cdflib_computefx(*p, *q, cum, ccum);
			if (status==0) 
			{
				break;
			}
			iteration=iteration+1;
		}
		if (status == 0) 
		{
			status = CDFLIB_OK;
		}
		else
		{
			cdflib_unabletoinvert("cdflib_cdffnc", *x, "x");
			status = CDFLIB_ERROR;
		}
		cdflib_printiter("cdflib_cdffnc", iteration);
	}
	return status;
}

int cdflib_fncCheckX(char * fname, double x)
{
	int status;
	status=cdflib_checkgreqthan(fname, x, "x", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}

int cdflib_fncCheckParams(char * fname, double dfn, double dfd, double phonc)
{
	int status;
	/*     DFN > 0 */
	status=cdflib_checkgreaterthan(fname, dfn, "dfn", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	/*     DFD > 0 */
	status=cdflib_checkgreaterthan(fname, dfd, "dfd", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	/*     PHONC >= 0 */
	status=cdflib_checkgreqthan(fname, phonc, "phonc", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}
